<?php

namespace FluentForm\App\Modules;

use FluentForm\App;
use FluentForm\View;

class AddOnModule
{
    /**
     * The number of days we'll cached the add-ons got from remote server.
     *
     * @var integer
     */
    protected $cacheDays = 1;

    /**
     * The URL to fetch the add-ons
     *
     * @var string
     */
    protected $addOnsFetchUrl = 'https://wpmanageninja.com/add-ons.json';

    /**
     * Render the add-ons list page.
     */
    public function render()
    {
        $extraMenus = [];

        $extraMenus = apply_filters('fluentform_addons_extra_menu', $extraMenus);

        $current_menu_item = 'fluentform_add_ons';

        if (isset($_GET['sub_page']) && $_GET['sub_page']) {
            $current_menu_item = sanitize_text_field($_GET['sub_page']);
        }

        wp_enqueue_style(
            'fluentform-add-ons',
            App::publicUrl('css/add-ons.css'),
            [],
	        FLUENTFORM_VERSION,
            'all'
        );

        return View::make('admin.addons.index', [
            'menus'             => $extraMenus,
            'base_url'          => admin_url('admin.php?page=fluent_form_add_ons'),
            'current_menu_item' => $current_menu_item
        ]);
    }

    /**
     * Show the add-ons list.
     */
    public function showFluentAddOns()
    {
    	$response = $this->getAddOns();
        View::render('admin.addons.list', [
            'addOns' => isset($response['add_ons']) ? $response['add_ons'] : array(),
	        'special_notice_top' => isset($response['special_notice_top']) ? $response['special_notice_top'] : '',
	        'special_notice_bottom' => isset($response['special_notice_bottom']) ? $response['special_notice_bottom'] : '',
        ]);
    }

    /**
     * Get the add-ons from the appropriate source
     * e.g. `options` table or, remote server.
     *
     * @return array $addOns
     */
    private function getAddOns()
    {
        $addOns = get_option('__fluentform_add_ons');

        if ($addOns) {
            // Since the add-ons are found from the options table
            // we should now check the expiry. If it's expired
            // we'll bust the cache requesting remote server.
            $cached = get_option('__fluentform_add_ons_caching_date');

            $now = date('Y-m-d H:i:s');

            $diff = (int) date_diff(new \DateTime($cached), new \DateTime($now))->format('%d');

            if ($diff > $this->cacheDays) {
                $addOns = $this->prepareAddOnsFromRemoteServer();
            }
        } else {
            // Since the add-ons are not present in the options
            // table we need to fetch it from the remote server.
            $addOns = $this->prepareAddOnsFromRemoteServer();
        }

        return $addOns;
    }

    /**
     * Request to the remote server to fetch the add-ons,
     * then store it to the options table for caching.
     *
     * @return array $addOns
     */
    private function prepareAddOnsFromRemoteServer()
    {
        $addOnsResponse = wp_remote_get($this->addOnsFetchUrl);

	    $error_message = false;
	    if ( is_wp_error( $addOnsResponse ) ) {
		    $error_message = $addOnsResponse->get_error_message();
	    }
	    
	    $addOnsJson = $addOnsResponse['body'];
	    
        $addOns = json_decode($addOnsJson, true);
        if($addOns) {
	        update_option('__fluentform_add_ons', $addOns);
	        update_option('__fluentform_add_ons_caching_date', date('Y-m-d H:i:s'));
        }
        return $addOns;
    }
}
