<?php

namespace FluentForm\App\Modules\Component;

use FluentForm\App\Modules\Acl\Acl;
use FluentForm\Framework\Foundation\Application;
use FluentForm\App\Services\FormBuilder\EditorShortcodeParser;
use FluentForm\App\Services\FormBuilder\Notifications\EmailNotificationActions;

class Component
{
    /**
     * FluentForm\Framework\Foundation\Application
     *
     * @var $app
     */
    protected $app = null;

    /**
     * Biuld the instance of this class
     *
     * @param \FluentForm\Framework\Foundation\Application $app
     */
    public function __construct(Application $app)
    {
        $this->app = $app;
    }

    /**
     * Get all the available components
     *
     * @return  void
     * @throws \Exception
     * @throws \FluentForm\Framework\Exception\UnResolveableEntityException
     */
    public function index()
    {
        Acl::verify('fluentform_forms_manager');

        $this->app->doAction(
            'fluent_editor_init',
            $components = $this->app->make('components')
        );

        $editorCompnents = $components->sort()->toArray();
        $editorCompnents = $this->app->applyFilters('fluent_editor_components', $editorCompnents);
        $countries = $this->app->load($this->app->appPath('Services/FormBuilder/CountryNames.php'));

        wp_send_json_success(array(
            'components'          => $editorCompnents,
            'countries'           => $countries,
            'disabled_components' => $this->getDisabledComponents()
        ));
        exit();
    }


    /**
     * Get disabled components
     *
     * @return array
     */
    private function getDisabledComponents()
    {
        $isReCaptchaDisabled = !get_option('_fluentform_reCaptcha_keys_status', false);

        $disabled = array(
            'recaptcha'    => array(
                'contentComponent' => 'recaptcha',
                'disabled'         => $isReCaptchaDisabled
            ),
            'input_image'  => array(
                'disabled' => true
            ),
            'input_file'   => array(
                'disabled' => true
            ),
            'input_repeat' => array(
                'disabled' => true
            ),
            'shortcode'    => array(
                'disabled' => true
            ),
            'action_hook'  => array(
                'disabled' => true
            ),
            'form_step'    => array(
                'disabled' => true
            )
        );

        if (!defined('FLUENTFORMPRO')) {
            $disabled['ratings'] = array(
                'disabled' => true
            );
        }
        return $this->app->applyFilters('fluentform_disabled_components', $disabled);
    }

    /**
     * Get available shortcodes for editor
     *
     * @return void
     * @throws \Exception
     */
    public function getEditorShortcodes()
    {
        Acl::verify('fluentform_forms_manager');
        $editor_shortcodes = fluentFormEditorShortCodes();
        wp_send_json_success(['shortcodes' => $editor_shortcodes], 200);
    }

    /**
     * Get all available shortcodes for editor
     *
     * @return void
     * @throws \Exception
     */
    public function getAllEditorShortcodes()
    {
        Acl::verify('fluentform_forms_manager');
        wp_send_json(fluentFormGetAllEditorShortCodes(
            $this->app->request->get('formId')
        ), 200);
    }

    /**
     * Register the form renderer shortcode
     *
     * @return  void
     */
    public function addFluentFormShortCode()
    {
        $this->app->addShortCode('fluentform', function ($atts, $content) {

            $this->registerScripts();

            $shortcodeDefaults = apply_filters('fluentform_shortcode_defaults', array(
                'id'                 => null,
                'title'              => null,
                'permission'         => '',
                'permission_message' => __('Sorry, You do not have permission to view this form', 'fluentform')
            ), $atts);

            $atts = shortcode_atts($shortcodeDefaults, $atts);

            $form_id = $atts['id'];

            if ($form_id) {
                $form = wpFluent()->table('fluentform_forms')->find($form_id);
            } else if ($formTitle = $atts['title']) {
                $form = wpFluent()->table('fluentform_forms')->where('title', $formTitle)->first();
            } else {
                return;
            }

            if (!$form) {
                return;
            }

            if ($atts['permission']) {
                if (!current_user_can($atts['permission'])) {
                    return "<div id='ff_form_{$form->id}' class='ff_form_not_render'>{$atts['permission_message']}</div>";
                }
            }

            $formSettings = wpFluent()
                ->table('fluentform_form_meta')
                ->where('form_id', $form_id)
                ->where('meta_key', 'formSettings')
                ->first();

            if (!$formSettings) {
                return;
            }

            $form->fields = json_decode($form->form_fields, true);

            if (!$form->fields['fields']) {
                return;
            }

            $form->settings = json_decode($formSettings->value, true);
            $form = $this->app->applyFilters('fluentform_rendering_form', $form);

            $isRenderable = array(
                'status'  => true,
                'message' => ''
            );

            $isRenderable = $this->app->applyFilters('fluentform_is_form_renderable', $isRenderable, $form);

            if (is_array($isRenderable) && !$isRenderable['status']) {
                return "<div id='ff_form_{$form->id}' class='ff_form_not_render'>{$isRenderable['message']}</div>";
            }

            $formBuilder = $this->app->make('formBuilder');
            $output = $formBuilder->build($form);
            $output = $this->processOutput($output, $form);

            wp_enqueue_style(
                'fluent-form-styles',
                $this->app->publicUrl('css/fluent-forms-public.css'),
                array(),
                FLUENTFORM_VERSION
            );

            wp_enqueue_style(
                'fluentform-public-default',
                $this->app->publicUrl('css/fluentform-public-default.css'),
                array(),
                FLUENTFORM_VERSION
            );

            wp_enqueue_script(
                'fluent-form-submission',
                $this->app->publicUrl('js/form-submission.js'),
                array('jquery'),
                FLUENTFORM_VERSION,
                true
            );

            $form_vars = array(
                'id'           => $form->id,
                'settings'     => $form->settings,
                'rules'        => $formBuilder->validationRules,
                'do_analytics' => $this->app->applyFilters('fluentform_do_analytics', true)
            );

            if ($conditionals = $formBuilder->conditions) {
                wp_enqueue_script(
                    'fluent-form-conditionals',
                    $this->app->publicUrl('js/form-conditionals.js'),
                    array('jquery'),
                    FLUENTFORM_VERSION,
                    true
                );
                $form_vars['conditionals'] = $conditionals;
            }

            wp_localize_script('fluent-form-submission', 'fluentFormVars', array(
                'ajaxUrl'            => admin_url('admin-ajax.php'),
                'forms'              => (Object)array(),
                'isDisableAnalytics' => apply_filters('fluentform-disabled_analytics', false),
                'date_i18n'          => $this->getDatei18n(),
                'pro_version'        => (defined('FLUENTFORMPRO_VERSION')) ?  FLUENTFORMPRO_VERSION : false,
                'fluentform_version' => FLUENTFORM_VERSION
            ));

            $this->addInlineVars(json_encode($form_vars), $form->id);

            return $output;

        });

        $this->app->addShortCode('fluentform_info', function ($atts) {
            $shortcodeDefaults = apply_filters('fluentform_info_shortcode_defaults', array(
                'id'    => null,
                'info' => 'submission_count',
                'status' => 'all',
                'with_trashed' => 'no',
                'substract_form' => 0,
                'hide_on_zero' => 'no',
                'date_format' => ''
            ), $atts);
            $atts = shortcode_atts($shortcodeDefaults, $atts);

            $formId = $atts['id'];

            $form = wpFluent()->table('fluentform_forms')->find($formId);

            if(!$form) {
                return '';
            }

            if($atts['info'] == 'submission_count') {
                $countQuery = wpFluent()->table('fluentform_submissions')
                                ->where('form_id', $formId);

                if($atts['status'] != 'trashed' && $atts['with_trashed'] == 'no') {
                    $countQuery = $countQuery->where('status', '!=', 'trashed');
                }

                if($atts['status'] == 'all') {
                } else if($atts['status'] == 'favourites') {
                    $countQuery = $countQuery->where('is_favourite', '=', 1);
                } else {
                    $countQuery = $countQuery->where('status', '=', sanitize_text_field($atts['status']));
                }

                $total = $countQuery->count();
                if($atts['substract_form']) {
                    $total = intval($atts['substract_form']) - $total;
                }

                if($atts['hide_on_zero'] && !$total) {
                    return '';
                }

                if($atts['hide_on_zero'] == 'yes' && !$total) {
                    return '';
                }

                return $total;
            } else if($atts['info'] == 'created_at') {
                if($atts['date_format']) {
                    $dateFormat = $atts['date_format'];
                } else {
                    $dateFormat = get_option( 'date_format' ) .' '. get_option( 'time_format' );
                }
                return date($dateFormat, strtotime($form->created_at));
            } else if($atts['info'] == 'updated_at') {
                if($atts['date_format']) {
                    $dateFormat = $atts['date_format'];
                } else {
                    $dateFormat = get_option( 'date_format' ) .' '. get_option( 'time_format' );
                }
                return date($dateFormat, strtotime($form->updated_at));
            }

            return '';
        });

    }


    /**
     * Register the Scripts that the inner components can use
     *
     * @return  void
     */
    private function registerScripts()
    {
        // Date Pickckr Style
        wp_register_style(
            'flatpickr',
            $this->app->publicUrl('libs/flatpickr/flatpickr.min.css')
        );

        // Date Pickckr Script
        wp_register_script(
            'flatpickr',
            $this->app->publicUrl('libs/flatpickr/flatpickr.js'),
            array('jquery'),
            false,
            true
        );

        wp_register_script(
            'selectWoo',
            $this->app->publicUrl('libs/selectWoo/selectWoo.full.min.js'),
            array('jquery'),
            '1.0.6',
            true
        );

        wp_register_style(
            'select2',
            $this->app->publicUrl('libs/selectWoo/select2.css')
        );


    }

    /**
     * Process the output HTML to generate the default values.
     *
     * @param string $output
     * @param \stdClass $form
     * @return void
     */
    private function processOutput($output, $form)
    {
        // Get the patterns for default values from the output HTML string.
        // The example of a pattern would be for user ID: {user.ID}
        preg_match_all('/{(.*?)}/', $output, $matches);
        $patterns = array_unique($matches[0]);

        $attrDefaultValues = [];

        foreach ($patterns as $pattern) {
            // The default value for each pattern will be resolved here.
            $attrDefaultValues[$pattern] = apply_filters('fluentform_parse_default_value', $pattern, $form);
        }
        // Raising an event so that others can hook into it and modify the default values later.
        $attrDefaultValues = (array)apply_filters('fluentform_parse_default_values', $attrDefaultValues);
        // Finally, replace the patterns with the replacements and return the output HTML.
        return str_replace(array_keys($attrDefaultValues), array_values($attrDefaultValues), $output);
    }

    /**
     * Register renderer actions for compiling each element
     *
     * @return  void
     */
    public function addRendererActions()
    {
        $actionMappings = [
            'Select@compile'        => ['fluentform_render_item_select'],
            'Rating@compile'        => ['fluentform_render_item_ratings'],
            'Address@compile'       => ['fluentform_render_item_address'],
            'Name@compile'          => ['fluentform_render_item_input_name'],
            'TextArea@compile'      => ['fluentform_render_item_textarea'],
            'DateTime@compile'      => ['fluentform_render_item_input_date'],
            'Recaptcha@compile'     => ['fluentform_render_item_recaptcha'],
            'Container@compile'     => ['fluentform_render_item_container'],
            'CustomHtml@compile'    => ['fluentform_render_item_custom_html'],
            'SectionBreak@compile'  => ['fluentform_render_item_section_break'],
            'SubmitButton@compile'  => ['fluentform_render_item_submit_button'],
            'SelectCountry@compile' => ['fluentform_render_item_select_country'],

            'TermsAndConditions@compile' => [
                'fluentform_render_item_terms_and_condition',
                'fluentform_render_item_gdpr_agreement'
            ],

            'TabularGrid@compile' => [
                'fluentform_render_item_tabular_grid'
            ],

            'Checkable@compile' => [
                'fluentform_render_item_input_radio',
                'fluentform_render_item_input_checkbox',
            ],

            'Text@compile' => [
                'fluentform_render_item_input_url',
                'fluentform_render_item_input_text',
                'fluentform_render_item_input_email',
                'fluentform_render_item_input_number',
                'fluentform_render_item_input_hidden',
                'fluentform_render_item_input_password',
            ],
        ];

        $path = 'FluentForm\App\Services\FormBuilder\Components\\';
        foreach ($actionMappings as $handler => $actions) {
            foreach ($actions as $action) {
                $this->app->addAction($action, function () use ($path, $handler) {
                    list($class, $method) = $this->app->parseHandler($path . $handler);
                    call_user_func_array(array($class, $method), func_get_args());
                }, 10, 2);
            }
        }
    }

    /**
     * Register dynamic value shortcode parser (filter default value)
     *
     * @return  void
     */
    public function addFluentFormDefaultValueParser()
    {
        $this->app->addFilter('fluentform_parse_default_value', function ($value, $form) {
            return EditorShortcodeParser::filter($value, $form);
        }, 10, 2);
    }

    /**
     * Register filter to check whether the form is renderable
     *
     * @return  mixed
     */
    public function addIsRenderableFilter()
    {
        $this->app->addFilter('fluentform_is_form_renderable', function ($isRenderable, $form) {
            $checkables = array('limitNumberOfEntries', 'scheduleForm', 'requireLogin');
            foreach ($form->settings['restrictions'] as $key => $restrictions) {
                if (in_array($key, $checkables)) {
                    $isRenderable['status'] = $this->{$key}($restrictions, $form, $isRenderable);
                    if (!$isRenderable['status']) {
                        return $isRenderable;
                    }
                }
            }

            return $isRenderable;
        }, 10, 2);
    }

    /**
     * Check if limit is set on form submits and it's valid yet
     *
     * @param  array $restrictions
     *
     * @return bool
     */
    private function limitNumberOfEntries($restrictions, $form, &$isRenderable)
    {
        if (!$restrictions['enabled']) {
            return true;
        }

        $col = 'created_at';
        $period = $restrictions['period'];
        $maxAllowedEntries = $restrictions['numberOfEntries'];
        $query = wpFluent()->table('fluentform_submissions')
            ->where('form_id', $form->id)
            ->where('status', '!=', 'trashed');

        if ($period == 'day') {
            $year = "YEAR(`{$col}`) = YEAR(NOW())";
            $month = "MONTH(`{$col}`) = MONTH(NOW())";
            $day = "DAY(`{$col}`) = DAY(NOW())";
            $query->where(wpFluent()->raw("{$year} AND {$month} AND {$day}"));
        } elseif ($period == 'week') {
            $query->where(
                wpFluent()->raw("YEARWEEK(`{$col}`, 1) = YEARWEEK(CURDATE(), 1)")
            );
        } elseif ($period == 'month') {
            $year = "YEAR(`{$col}`) = YEAR(NOW())";
            $month = "MONTH(`{$col}`) = MONTH(NOW())";
            $query->where(wpFluent()->raw("{$year} AND {$month}"));
        } elseif ($period == 'year') {
            $query->where(wpFluent()->raw("YEAR(`{$col}`) = YEAR(NOW())"));
        }

        if (!($isAllowed = ($query->count() < $maxAllowedEntries))) {
            $isRenderable['message'] = $restrictions['limitReachedMsg']
                ? $restrictions['limitReachedMsg']
                : 'Maximum number of entries exceeded.';
        }

        return $isAllowed;
    }

    /**
     * Check if form has scheduled date and open for submission
     *
     * @param  array $restrictions
     *
     * @return bool
     */
    private function scheduleForm($restrictions, $form, &$isRenderable)
    {
        if (!$restrictions['enabled']) {
            return true;
        }

        $time = time();
        $start = strtotime($restrictions['start']);
        $end = strtotime($restrictions['end']);

        if ($time < $start) {
            $isRenderable['message'] = $restrictions['pendingMsg']
                ? $restrictions['pendingMsg']
                : 'Form submission is not started yet.';

            return false;
        }

        if ($time >= $end) {
            $isRenderable['message'] = $restrictions['expiredMsg']
                ? $restrictions['expiredMsg']
                : 'Form submission is now closed.';

            return false;
        }

        return true;
    }

    /**
     * * Check if form requires loged in user and user is logged in
     *
     * @param  array $restrictions
     *
     * @return bool
     */
    private function requireLogin($restrictions, $form, &$isRenderable)
    {
        if (!$restrictions['enabled']) {
            return true;
        }

        if (!($isLoggedIn = is_user_logged_in())) {
            $isRenderable['message'] = $restrictions['requireLoginMsg']
                ? $restrictions['requireLoginMsg']
                : 'You must be logged in to submit the form.';
        }

        return $isLoggedIn;
    }

    /**
     * Register fluentform_submission_inserted actions
     *
     * @return  void
     */
    public function addFluentformSubmissionInsertedFilter()
    {
        if (!fluentFormIsHandlingSubmission()) {
            return;
        }

        (new EmailNotificationActions($this->app))->register();
    }

    /**
     * Add inline scripts [Add localized script using same var]
     *
     * @param array $vars
     * @param int $form_id
     *
     * @return  void
     */
    private function addInlineVars($vars, $form_id)
    {
        add_action('wp_footer', function () use ($vars, $form_id) {
            ?>
            <script type="text/javascript">
                window.fluentFormVars.forms["fluentform_<?php echo $form_id;?>"] = <?php echo $vars; ?>;
            </script>
            <?php
        }, 100);
    }

    private function getDatei18n()
    {
        $i18n = array(
            'previousMonth'    => __('Previous Month', 'fluentform'),
            'nextMonth'        => __('Next Month', 'fluentform'),
            'months'           => [
                'shorthand' => [
                    __('Jan', 'fluentform'),
                    __('Feb', 'fluentform'),
                    __('Mar', 'fluentform'),
                    __('Apr', 'fluentform'),
                    __('May', 'fluentform'),
                    __('Jun', 'fluentform'),
                    __('Jul', 'fluentform'),
                    __('Aug', 'fluentform'),
                    __('Sep', 'fluentform'),
                    __('Oct', 'fluentform'),
                    __('Nov', 'fluentform'),
                    __('Dec', 'fluentform')
                ],
                'longhand'  => [
                    __('January', 'fluentform'),
                    __('February', 'fluentform'),
                    __('March', 'fluentform'),
                    __('April', 'fluentform'),
                    __('May', 'fluentform'),
                    __('June', 'fluentform'),
                    __('July', 'fluentform'),
                    __('August', 'fluentform'),
                    __('September', 'fluentform'),
                    __('October', 'fluentform'),
                    __('November', 'fluentform'),
                    __('December', 'fluentform')
                ]
            ],
            'weekdays'         => [
                'longhand'  => array(
                    __('Sunday', 'fluentform'),
                    __('Monday', 'fluentform'),
                    __('Tuesday', 'fluentform'),
                    __('Wednesday', 'fluentform'),
                    __('Thursday', 'fluentform'),
                    __('Friday', 'fluentform'),
                    __('Saturday', 'fluentform')
                ),
                'shorthand' => array(
                    __('Sun', 'fluentform'),
                    __('Mon', 'fluentform'),
                    __('Tue', 'fluentform'),
                    __('Wed', 'fluentform'),
                    __('Thu', 'fluentform'),
                    __('Fri', 'fluentform'),
                    __('Sat', 'fluentform')
                )
            ],
            'daysInMonth'      => [
                31,
                28,
                31,
                30,
                31,
                30,
                31,
                31,
                30,
                31,
                30,
                31
            ],
            'rangeSeparator'   => __(' to ', 'fluentform'),
            'weekAbbreviation' => __('Wk', 'fluentform'),
            'scrollTitle'      => __('Scroll to increment', 'fluentform'),
            'toggleTitle'      => __('Click to toggle', 'fluentform'),
            'amPM'             => [
                __('AM', 'fluentform'),
                __('PM', 'fluentform')
            ],
            'yearAriaLabel'    => __('Year', 'fluentform')
        );

        return apply_filters('fluentform/date_i18n', $i18n);
    }
}
