<?php

namespace FluentForm\App\Services\FormBuilder\Notifications;

use FluentForm\App\Services\Emogrifier\Emogrifier;
use FluentForm\Framework\Foundation\Application;
use FluentForm\App\Modules\Form\FormFieldsParser;
use FluentForm\View;

class EmailNotification
{
    /**
     * FluentForm\Framework\Foundation\Application
     * @var $app
     */
    protected $app = null;

    /**
     * Biuld the instance of this class
     * @param  FluentForm\Framework\Foundation\Application $app
     * @return $this
     */
    public function __construct(Application $app)
    {
        $this->app = $app;
    }

    /**
     * Send the email notification
     * @param  array    $notification  [Notification settings from form meta]
     * @param  array    $submittedData [User submitted form data]
     * @param  StdClass $form          [The form object from database]
     * @return bool
     */
    public function notify($notification, $submittedData, $form, $entryId = false)
    {
        $headers = [
            'Content-Type: text/html; charset=UTF-8'
        ];

        if ($notification['fromName'] && $notification['fromEmail']) {
            $headers[] = "From: {$notification['fromName']} <{$notification['fromEmail']}>";
        } elseif ($notification['fromName']) {
            $headers[] = "From: {$notification['fromName']}";
        } elseif ($notification['fromEmail']) {
            $headers[] = "From: <{$notification['fromEmail']}>";
        }

        if ($notification['replyTo']) {
            $headers[] = "Reply-To: <{$notification['replyTo']}>";
        }

        $headers = $this->app->applyFilters(
            'fluenttform_email_header', $headers, $notification
        );

        $attachments = $this->app->applyFilters(
            'fluentform_filter_email_notification',
            isset($notification['attachments']) ? $notification['attachments'] : [],
            $notification,
            $form,
            $submittedData
        );
        $emailBody = $notification['message'];

        if(!apply_filters('fluenform_send_plain_html_email', false, $form, $notification)) {
            $emailBody = $this->getEmailWithTemplate($emailBody, $form, $notification);
        }

        if(!$notification['sendTo']['email'] || !$notification['subject']) {
            return false;
        }

        $isMailSentSuccessfully = wp_mail(
            $notification['sendTo']['email'],
            $notification['subject'],
            $emailBody,
            $headers,
            $attachments
        );

        $this->emptyTmp($attachments);

        if($entryId) {
            if($isMailSentSuccessfully) {
                $message = 'Email Notification sent to '.$notification['sendTo']['email'].' and the subject: '.$notification['subject'];
            } else {
                $message = 'Email Notification may failed to deliver to '.$notification['sendTo']['email'].' and the subject: '.$notification['subject'].', Please check your SMTP/wp_mail functionality/configration';
            }

            wpFluent()->table('fluentform_submission_meta')
                ->insert([
                    'response_id' => $entryId,
                    'form_id' => $form->id,
                    'meta_key' => 'api_log',
                    'value' => $message,
                    'status' => 'info',
                    'name' => 'FluentForm Bot',
                    'created_at' => date('Y-m-d H:i:s'),
                    'updated_at' => date('Y-m-d H:i:s')
                ]);
        }

        return $isMailSentSuccessfully;
    }

    /**
     * Delete attached files from tmp directory
     * @param  array $attachments
     * @return void
     */
    protected function emptyTmp($attachments)
    {
        if ($attachments) {
            foreach ($attachments as $path) unlink($path);
        }
    }

    /**
     * @param $formId
     * @todo: Implement Caching mechanism so we don't have to parse these things for every request
     * @return array
     */
    private function getFormInputsAndLabels($form)
    {
        $formInputs = FormFieldsParser::getInputs($form);

        $inputLabels = FormFieldsParser::getAdminLabels($form, $formInputs);

        return [
            'inputs' => $formInputs,
            'labels' => $inputLabels
        ];
    }

    public function getEmailWithTemplate($emailBody, $form, $notification)
    {
        $emailHeader = apply_filters('fluentform_email_header', '', $form, $notification);
        $emailFooter = apply_filters('fluentform_email_footer', '', $form, $notification);

        if(empty($emailHeader)) {
            $emailHeader = View::make('email.template.header', array(
                'form' => $form,
                'notification' => $notification
            ));
        }

        if(empty($emailFooter)) {
            $emailFooter = View::make('email.template.footer', array(
                'form' => $form,
                'notification' => $notification
            ));
        }

        $css = View::make('email.template.styles');
        $css = apply_filters('fluentform_email_styles', $css, $form, $notification);
        $emailBody = $emailHeader.$emailBody.$emailFooter;

        try {
            // apply CSS styles inline for picky email clients
            $emogrifier = new Emogrifier($emailBody, $css);
            $emailBody = $emogrifier->emogrify();

        } catch ( Exception $e ) {

        }
        return $emailBody;
    }
}
